@@ -1,4 +1,4 @@
-/* Copyright (c) 2007-2017, The Tor Project, Inc. */
+/* Copyright (c) 2007-2019, The Tor Project, Inc. */
 /* See LICENSE for licensing information */
 
 /**
@@ -12,16 +12,27 @@
 #define RENDSERVICE_PRIVATE
 #define HS_SERVICE_PRIVATE
 
-#include "or.h"
-#include "test.h"
-#include "control.h"
-#include "config.h"
-#include "hs_common.h"
-#include "rendcommon.h"
-#include "rendservice.h"
-#include "routerset.h"
-#include "circuitbuild.h"
-#include "test_helpers.h"
+#include "core/or/or.h"
+#include "test/test.h"
+#include "feature/control/control.h"
+#include "app/config/config.h"
+#include "feature/hs/hs_common.h"
+#include "feature/rend/rendcommon.h"
+#include "feature/rend/rendservice.h"
+#include "feature/nodelist/routerlist.h"
+#include "feature/nodelist/routerset.h"
+#include "core/or/circuitbuild.h"
+
+#include "feature/nodelist/node_st.h"
+#include "feature/rend/rend_encoded_v2_service_descriptor_st.h"
+#include "feature/rend/rend_intro_point_st.h"
+#include "feature/nodelist/routerinfo_st.h"
+
+#include "test/test_helpers.h"
+
+#ifdef HAVE_UNISTD_H
+#include <unistd.h>
+#endif
 
 /* mock ID digest and longname for node that's in nodelist */
 #define HSDIR_EXIST_ID "\xAA\xAA\xAA\xAA\xAA\xAA\xAA\xAA\xAA\xAA" \
@@ -343,76 +354,6 @@ test_hs_desc_event(void *arg)
   tor_free(received_msg);
 }
 
-/* Make sure we always pick the right RP, given a well formatted
- * Tor2webRendezvousPoints value. */
-static void
-test_pick_tor2web_rendezvous_node(void *arg)
-{
-  or_options_t *options = get_options_mutable();
-  const node_t *chosen_rp = NULL;
-  router_crn_flags_t flags = CRN_NEED_DESC;
-  int retval, i;
-  const char *tor2web_rendezvous_str = "test003r";
-
-  (void) arg;
-
-  /* Setup fake routerlist. */
-  helper_setup_fake_routerlist();
-
-  /* Parse Tor2webRendezvousPoints as a routerset. */
-  options->Tor2webRendezvousPoints = routerset_new();
-  options->UseMicrodescriptors = 0;
-  retval = routerset_parse(options->Tor2webRendezvousPoints,
-                           tor2web_rendezvous_str,
-                           "test_tor2web_rp");
-  tt_int_op(retval, OP_GE, 0);
-
-  /* Pick rendezvous point. Make sure the correct one is
-     picked. Repeat many times to make sure it works properly. */
-  for (i = 0; i < 50 ; i++) {
-    chosen_rp = pick_tor2web_rendezvous_node(flags, options);
-    tt_assert(chosen_rp);
-    tt_str_op(chosen_rp->ri->nickname, OP_EQ, tor2web_rendezvous_str);
-  }
-
- done:
-  routerset_free(options->Tor2webRendezvousPoints);
-}
-
-/* Make sure we never pick an RP if Tor2webRendezvousPoints doesn't
- * correspond to an actual node. */
-static void
-test_pick_bad_tor2web_rendezvous_node(void *arg)
-{
-  or_options_t *options = get_options_mutable();
-  const node_t *chosen_rp = NULL;
-  router_crn_flags_t flags = CRN_NEED_DESC;
-  int retval, i;
-  const char *tor2web_rendezvous_str = "dummy";
-
-  (void) arg;
-
-  /* Setup fake routerlist. */
-  helper_setup_fake_routerlist();
-
-  /* Parse Tor2webRendezvousPoints as a routerset. */
-  options->Tor2webRendezvousPoints = routerset_new();
-  retval = routerset_parse(options->Tor2webRendezvousPoints,
-                           tor2web_rendezvous_str,
-                           "test_tor2web_rp");
-  tt_int_op(retval, OP_GE, 0);
-
-  /* Pick rendezvous point. Since Tor2webRendezvousPoints was set to a
-     dummy value, we shouldn't find any eligible RPs. */
-  for (i = 0; i < 50 ; i++) {
-    chosen_rp = pick_tor2web_rendezvous_node(flags, options);
-    tt_ptr_op(chosen_rp, OP_EQ, NULL);
-  }
-
- done:
-  routerset_free(options->Tor2webRendezvousPoints);
-}
-
 /* Make sure rend_data_t is valid at creation, destruction and when
  * duplicated. */
 static void
@@ -1035,11 +976,6 @@ struct testcase_t hs_tests[] = {
     NULL, NULL },
   { "hs_desc_event", test_hs_desc_event, TT_FORK,
     NULL, NULL },
-  { "pick_tor2web_rendezvous_node", test_pick_tor2web_rendezvous_node, TT_FORK,
-    NULL, NULL },
-  { "pick_bad_tor2web_rendezvous_node",
-    test_pick_bad_tor2web_rendezvous_node, TT_FORK,
-    NULL, NULL },
   { "hs_auth_cookies", test_hs_auth_cookies, TT_FORK,
     NULL, NULL },
   { "single_onion_poisoning_create_dir_none", test_single_onion_poisoning,
@@ -1055,4 +991,3 @@ struct testcase_t hs_tests[] = {
 
   END_OF_TESTCASES
 };
-
